@@ -1,6 +1,10 @@
+from .async_eventbus_rabbitmq import AsyncEventbusRabbitMQ
+from .async_connection_factory import AsyncConnectionFactoryRabbitMQ
+from .async_connection import AsyncConnection
+from .async_channel_factory import AsyncChannelFactoryRabbitMQ
+from .async_channel import AsyncChannel
+from .eventbus_rabbitmq import EventbusRabbitMQ
 from .connection_factory_rabbitmq import ConnectionFactoryRabbitMQ
 from .connection_rabbitmq import ConnectionRabbitMQ
 from .channel_factory_rabbitmq import ChannelFactoryRabbitMQ
 from .channel_rabbitmq import ChannelRabbitMQ
-from .eventbus_rabbitmq import EventbusRabbitMQ
-from .async_eventbus_rabbitmq import AsyncEventbusRabbitMQ
\ No newline at end of file
